##########################################################################
#
# pgAdmin 4 - PostgreSQL Tools
#
# Copyright (C) 2013 - 2018, The pgAdmin Development Team
# This software is released under the PostgreSQL Licence
#
##########################################################################

import uuid
import json

from pgadmin.browser.server_groups.servers.databases.schemas.tables.tests \
    import utils as tables_utils
from pgadmin.browser.server_groups.servers.databases.schemas.tests import \
    utils as schema_utils
from pgadmin.browser.server_groups.servers.databases.tests import utils as \
    database_utils
from pgadmin.utils.route import BaseTestGenerator
from regression import parent_node_dict
from regression.python_test_utils import test_utils as utils
from . import utils as columns_utils


class ColumnDeleteMultipleTestCase(BaseTestGenerator):
    """This class will delete column under table node."""
    scenarios = [
        ('Delete column Node URL', dict(url='/browser/column/obj/'))
    ]

    def setUp(self):
        self.db_name = parent_node_dict["database"][-1]["db_name"]
        schema_info = parent_node_dict["schema"][-1]
        self.server_id = schema_info["server_id"]
        self.db_id = schema_info["db_id"]
        db_con = database_utils.connect_database(self, utils.SERVER_GROUP,
                                                 self.server_id, self.db_id)
        if not db_con['data']["connected"]:
            raise Exception("Could not connect to database to add a table.")
        self.schema_id = schema_info["schema_id"]
        self.schema_name = schema_info["schema_name"]
        schema_response = schema_utils.verify_schemas(self.server,
                                                      self.db_name,
                                                      self.schema_name)
        if not schema_response:
            raise Exception("Could not find the schema to add a table.")
        self.table_name = "table_column_%s" % (str(uuid.uuid4())[1:8])
        self.table_id = tables_utils.create_table(self.server, self.db_name,
                                                  self.schema_name,
                                                  self.table_name)
        self.column_name = "test_column_delete_%s" % (str(uuid.uuid4())[1:8])
        self.column_name_1 = "test_column_delete_%s" % (str(uuid.uuid4())[1:8])
        self.column_ids = [columns_utils.create_column(self.server,
                                                       self.db_name,
                                                       self.schema_name,
                                                       self.table_name,
                                                       self.column_name),
                           columns_utils.create_column(self.server,
                                                       self.db_name,
                                                       self.schema_name,
                                                       self.table_name,
                                                       self.column_name_1)
                           ]

    def runTest(self):
        """This function will drop column under table node."""
        col_response = columns_utils.verify_column(self.server, self.db_name,
                                                   self.column_name)
        if not col_response:
            raise Exception("Could not find the column to drop.")

        col_response = columns_utils.verify_column(self.server, self.db_name,
                                                   self.column_name_1)
        if not col_response:
            raise Exception("Could not find the column to drop.")

        data = {'ids': self.column_ids}
        response = self.tester.delete(self.url + str(utils.SERVER_GROUP) +
                                      '/' + str(self.server_id) + '/' +
                                      str(self.db_id) + '/' +
                                      str(self.schema_id) + '/' +
                                      str(self.table_id) + '/',
                                      follow_redirects=True,
                                      data=json.dumps(data),
                                      content_type='html/json'
                                      )
        self.assertEquals(response.status_code, 200)

    def tearDown(self):
        # Disconnect the database
        database_utils.disconnect_database(self, self.server_id, self.db_id)
